//
//    Title: clip.cc
//  Created: Thu Nov  1 17:23:43 2001
//   Author: Tim Weyrich <weyrich@inf.ethz.ch>
//
// copyright (c) 2001--2003, Computer Graphics Laboratory, ETH Zuerich
//

#include <math.h>

#include "clip.h"

///////////////////////////////////////////////////////////////////////////////

int clippingCode(double x, double y, double xcl1, double ycl1, double xcl2, double ycl2)
{
  //   Compute the endpoint codes for clip().
  int code = 0;
  if (x < xcl1) code = code | 0x1;
  if (x > xcl2) code = code | 0x2;
  if (y < ycl1) code = code | 0x4;
  if (y > ycl2) code = code | 0x8;
  return code;
}

int clip(float *x, float *y, float xclipl, float yclipb, float xclipr, float yclipt)
{
  //   Clipping routine: Cohen Sutherland algorithm.
  // If Clip ==2 the segment is outside the boundary.
  // If Clip ==1 the segment has one point outside the boundary.
  // If Clip ==0 the segment is inside the boundary.
  //
  // _Input parameters:
  //
  //  x[2], y[2] : Segment coordinates
  //  xclipl, yclipb, xclipr, yclipt : Clipping boundary
  //
  // _Output parameters:
  //
  //  x[2], y[2] : New segment coordinates
  //
  const float P=10000;
  int clip = 0;

  for (int i=0;i<2;i++) {
    if (fabs(xclipl-x[i]) <= fabs(xclipr-xclipl)/P) x[i] = xclipl;
    if (fabs(xclipr-x[i]) <= fabs(xclipr-xclipl)/P) x[i] = xclipr;
    if (fabs(yclipb-y[i]) <= fabs(yclipt-yclipb)/P) y[i] = yclipb;
    if (fabs(yclipt-y[i]) <= fabs(yclipt-yclipb)/P) y[i] = yclipt;
  }

  //Compute the first endpoint codes.
  int code1 = clippingCode(x[0],y[0],xclipl,yclipb,xclipr,yclipt);
  int code2 = clippingCode(x[1],y[1],xclipl,yclipb,xclipr,yclipt);

  double xt=0, yt=0;
  int clipped = 0; //this variable could be used in a future version
  while(code1 + code2) {
    clipped = 1;

    //The line lies entirely outside the clipping boundary

    if (code1&code2) {
      clip = 2;
      return clip;
    }
    //The line is subdivided into several parts

    int ic = code1;
    if (ic == 0) ic = code2;
    if (ic & 0x1) {
      yt = y[0] + (y[1]-y[0])*(xclipl-x[0])/(x[1]-x[0]);
      xt = xclipl;
    }
    if (ic & 0x2) {
      yt = y[0] + (y[1]-y[0])*(xclipr-x[0])/(x[1]-x[0]);
      xt = xclipr;
    }
    if (ic & 0x4) {
      xt = x[0] + (x[1]-x[0])*(yclipb-y[0])/(y[1]-y[0]);
      yt = yclipb;
    }
    if (ic & 0x8) {
      xt = x[0] + (x[1]-x[0])*(yclipt-y[0])/(y[1]-y[0]);
      yt = yclipt;
    }
    if (ic == code1) {
      x[0]  = (float)xt;
      y[0]  = (float)yt;
      code1 = clippingCode(xt,yt,xclipl,yclipb,xclipr,yclipt);
    } else {
      x[1]  = (float)xt;
      y[1]  = (float)yt;
      code2 = clippingCode(xt,yt,xclipl,yclipb,xclipr,yclipt);
    }
  }
  clip = clipped;
  return clip;
}

///////////////////////////////////////////////////////////////////////////////
